﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Live2D.Cubism.Core;
using Live2D.Cubism.Framework;

public enum Parts { Head = 0, Body = 1, LeftArm = 2, RightArm = 3, Leg = 4}

//[RequireComponent(typeof(SpriteRenderer))]
public class PartsStatus :MonoBehaviour
{
    /// <summary>
    /// パーツステータス
    /// </summary>
    [System.Serializable]
    public class PartsList
    {
        public Parts Parts_Type; //パーツタイプ
        public string PartsName; //パーツ名
        public Sprite PartsRenderer; //パーツ画像
        public int HP_Parameter; //パーツ内臓HP
        public int Attack_Parameter; //パーツ内臓攻撃力
        public int Defense_Parameter; //パーツ内臓防御力
        public int Speed_Parameter; //パーツ内臓速度
    }

    public PartsList Part;
    
    /// <summary>
    /// Live2D
    /// </summary>
    [SerializeField]
    private CubismModel Model;
    private float Parameter = 1;

    private void Start()
    {
        //パーツ表示用に空にしておく
        if (this.gameObject.GetComponent<SpriteRenderer>().sprite != null)
        {
            this.gameObject.GetComponent<SpriteRenderer>().sprite = null;
        }
    }

    private void Update()
    {
        
    }

    private void LateUpdate()
    {
        //Live2D 岩のモデル
        var StoneEnable = Model.Parameters[27];
        ///岩の耐久値
        StoneEnable.Value = Parameter;
        ///岩の耐久値が0になったら
        if (Model.Parameters[27].Value <= 0.1f)
        {
            //パーツ表示
            this.gameObject.GetComponent<SpriteRenderer>().sprite = Part.PartsRenderer;
        }
    }

    private void OnTriggerEnter2D(Collider2D other)
    {

        //弾にヒット時
        if (other.gameObject.tag == "Bullet_tag")
        {
            //初期値5回攻撃したら岩が全部砕ける計算
            Parameter -= 0.2f;
        }

        //岩が剥けた状態での接触時
        if (other.gameObject.tag == "Player" && Model.Parameters[27].Value <= 0.1f)
        {
            other.GetComponent<PlayerSE>().PlayPowerUP();
            PlayerController playerController = other.gameObject.GetComponent<PlayerController>();

            switch (Part.Parts_Type)
            {
                case Parts.Head:
                    other.transform.Find("Head").GetComponent<SpriteRenderer>().sprite = Part.PartsRenderer;
                    playerController.shotIntervalFlame -=
                        playerController.shotIntervalFlame > 1 ? 5 : 0;
                    break;
                case Parts.Body:
                    other.transform.Find("Body").GetComponent<SpriteRenderer>().sprite = Part.PartsRenderer;
                    playerController.MaxHp += 50;
                    playerController.HP += 50;
                    break;
                case Parts.LeftArm:
                    other.transform.Find("Left").GetComponent<SpriteRenderer>().sprite = Part.PartsRenderer;
                    playerController.bulletCount += 1;
                    break;
                case Parts.RightArm:
                    other.transform.Find("Right").GetComponent<SpriteRenderer>().sprite = Part.PartsRenderer;
                    playerController.bulletCount += 1;
                    break;
                case Parts.Leg:
                    other.transform.Find("Bottom").GetComponent<SpriteRenderer>().sprite = Part.PartsRenderer;
                    //取得したプレイヤーの速度上げ
                    playerController.increaseSpeed = Part.Speed_Parameter;
                    break;
            }
            this.gameObject.SetActive(false);
        }
        
        //if (other.gameObject.tag == "Player" && Model.Parameters[27].Value > 0)
        //{
        //    ///ダメージ処理
        //    //Destroy(this.gameObject);
        //}

    }
}