﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerController : MonoBehaviour
{
    
    public CameraShake shake;
    public Bullet m_shotPrefab; // 弾のプレハブ
    // プレイヤーの移動スピード
    public const float BaseSpeed = 1;
    //変動するプレイヤーの移動速度
    public float increaseSpeed;

    //弾速
    public float bulletSpeed = 0.2f;
    //弾数
    public int bulletCount = 1;
    //ショット発射間隔フレーム
    public int shotIntervalFlame = 20;
    int shotIntervalTimer = 0;

    //弾の発射角度のベース 画面の向きが変わる場合これを弄ればOKです
    public int bulletAngleBase = 90;

    //1発ごとの拡散角度
    public int bulletSpreadAngle = 15;

    //ショットの攻撃力
    public int bulletPower = 3;
    //HP
    
    public float HP = 100.0f;
    public const float initialHP = 100.0f;
    public float MaxHp = initialHP;
    //死んでいるフラグ
    public bool deadFlag = false;

    public bool battleFlag = false;
    //自機の番号
    [SerializeField, Range(1, 2)] public int playerNumber = 1;

    // Start is called before the first frame update
    void Start()
    {
        //初期速度の取得
        increaseSpeed = BaseSpeed;
        //HPを初期化
        HP = initialHP;
        deadFlag = false;
    }


    //自機を操作するぞ
    void Update()
    {

        Transform myTransform = this.transform;
        var pos = myTransform.position; // プレイヤーの位置
        // 右・左 文字列の連結を行っている。これはHorizontal_1pみたいなことをしている
        //Unity上のEdit -> Project Setting -> Input でこれと同名のものを見て反応する
        float x = Input.GetAxisRaw("Horizontal_" + playerNumber + "p");
        // 上・下
        float y = Input.GetAxisRaw("Vertical_" + playerNumber + "p");
        // 移動する向きを求める
        Vector2 direction = new Vector2(x, y).normalized;

        // 移動する向きとスピードを代入する
        GetComponent<Rigidbody2D>().velocity = direction * (increaseSpeed + BaseSpeed);

        Vector3 mypos = myTransform.position;
        mypos.x = Mathf.Clamp(mypos.x, -3.5f, 3.5f);
        mypos.y = Mathf.Clamp(mypos.y, -2.0f, 2.0f);

        //画面外判定
        if (battleFlag == true && playerNumber == 1)
        {
            mypos.x = Mathf.Clamp(mypos.x, -8.5f, -0.8f);
        }
        if (battleFlag == true && playerNumber == 2)
        {
            mypos.x = Mathf.Clamp(mypos.x, 0.5f, 8.5f);
        }
        myTransform.position = mypos;

        shotIntervalTimer--;
        if (Input.GetButton("Fire1_" + playerNumber + "p") && shotIntervalTimer <= 0)
        {
            shotIntervalTimer = shotIntervalFlame;
            int count = bulletCount;
            for (int i = 1; i <= count; i++)
            {
                var angle = 0.0f;
                //0除算回避 (count - 1)のcountが1の場合フリーズする可能性があるため
                angle = -(count + 1) * bulletSpreadAngle / 2 + i * bulletSpreadAngle + bulletAngleBase;

                var shot = Instantiate(m_shotPrefab, pos, Quaternion.identity);

                shot.Init(angle, bulletSpeed, bulletPower, playerNumber);
            }
            GetComponent<PlayerSE>().PlayShot();

        }
    }

    private void OnTriggerEnter2D(Collider2D collision)
    {
        // 弾と衝突した場合
        if (collision.gameObject.tag == "Bullet_tag")
        {
            var obj = collision.gameObject.GetComponent<Bullet>();
            if (obj.shotChara != playerNumber)
            {
                //プレイヤー同士の弾の衝突判定 バトルモードに入るとお互いに当たるようになる
                if (battleFlag == true && (playerNumber == 1 || playerNumber == 2))
                {
                    Debug.Log(obj.shotChara + "Pの弾が" + playerNumber + "Pに命中。" + obj.power + "ダメージ");

                    HpDecrease(obj.power);
                    // 弾を削除する
                    Destroy(collision.gameObject);
                }
            }
        }
    }

    //HPを減算する
    private void HpDecrease(int num)
    {
        this.HP -= num;
        GetComponent<PlayerSE>().PlayHit();

        //HPが減ったタイミングで0以下になった場合は死んでいるので
        if (this.HP <= 0 && deadFlag == false)
        {
            this.HP = 0;
            deadFlag = true;
            shake.Shake(0.2f, 0.3f); //カメラを揺らす
            
            Destroy(gameObject, 3.0f);
            GetComponent<PlayerSE>().PlayDeath();
        }
    }
}